package org.netbeans.lib.cvsclient.file;

/**
 * @author Thomas Singer
 */
public final class FilePermissions {

	// Static =================================================================

	public static final FilePermissions DEFAULT_READABLE = new FilePermissions(true, false, false,
	                                                                           true, false, false,
	                                                                           true, false, false);
	public static final FilePermissions DEFAULT_WRITABLE = new FilePermissions(true, true, false,
	                                                                           true, false, false,
	                                                                           true, false, false);

	// u=rw,g=rw,o=rw
	public static FilePermissions parseCvsString(String mode) {
		boolean userReadable = false;
		boolean userWritable = false;
		boolean userExecutable = false;
		boolean groupReadable = false;
		boolean groupWritable = false;
		boolean groupExecutable = false;
		boolean worldReadable = false;
		boolean worldWritable = false;
		boolean worldExecutable = false;

		if (mode == null) {
			mode = "";
		}

		char group = ' ';
		for (int i = 0; i < mode.length(); i++) {
			final char chr = mode.charAt(i);
			if (chr == 'u' || chr == 'g' || chr == 'o') {
				group = chr;
				continue;
			}

			if (chr == ',') {
				group = ' ';
				continue;
			}

			if (chr == 'r') {
				if (group == 'u') {
					userReadable = true;
				}
				else if (group == 'g') {
					groupReadable = true;
				}
				else if (group == 'w') {
					worldReadable = true;
				}
				continue;
			}

			if (chr == 'w') {
				if (group == 'u') {
					userWritable = true;
				}
				else if (group == 'g') {
					groupWritable = true;
				}
				else if (group == 'w') {
					worldWritable = true;
				}
				continue;
			}

			if (chr == 'x') {
				if (group == 'u') {
					userExecutable = true;
				}
				else if (group == 'g') {
					groupExecutable = true;
				}
				else if (group == 'w') {
					worldExecutable = true;
				}
			}
		}
		return new FilePermissions(userReadable, userWritable, userExecutable,
		                           groupReadable, groupWritable, groupExecutable,
		                           worldReadable, worldWritable, worldExecutable);
	}

	public static FilePermissions parseBitRepresentation(String bitReprentation) {
		if (bitReprentation.length() < 10) {
			return DEFAULT_WRITABLE;
		}

		final boolean userReadable = bitReprentation.charAt(1) == 'r';
		final boolean userWritable = bitReprentation.charAt(2) == 'w';
		final boolean userExecutable = bitReprentation.charAt(3) == 'x';
		final boolean groupReadable = bitReprentation.charAt(4) == 'r';
		final boolean groupWritable = bitReprentation.charAt(5) == 'w';
		final boolean groupExecutable = bitReprentation.charAt(6) == 'x';
		final boolean worldReadable = bitReprentation.charAt(7) == 'r';
		final boolean worldWritable = bitReprentation.charAt(8) == 'w';
		final boolean worldExecutable = bitReprentation.charAt(9) == 'x';
		return new FilePermissions(userReadable, userWritable, userExecutable,
		                           groupReadable, groupWritable, groupExecutable,
		                           worldReadable, worldWritable, worldExecutable);
	}

	// Fields =================================================================

	private final boolean userReadable;
	private final boolean userWritable;
	private final boolean userExecutable;
	private final boolean groupReadable;
	private final boolean groupWritable;
	private final boolean groupExecutable;
	private final boolean worldReadable;
	private final boolean worldWritable;
	private final boolean worldExecutable;

	// Setup ==================================================================

	private FilePermissions(boolean userReadable, boolean userWritable, boolean userExecutable,
	                        boolean groupReadable, boolean groupWritable, boolean groupExecutable,
	                        boolean worldReadable, boolean worldWritable, boolean worldExecutable) {
		this.userReadable = userReadable;
		this.userWritable = userWritable;
		this.userExecutable = userExecutable;
		this.groupReadable = groupReadable;
		this.groupWritable = groupWritable;
		this.groupExecutable = groupExecutable;
		this.worldReadable = worldReadable;
		this.worldWritable = worldWritable;
		this.worldExecutable = worldExecutable;
	}

	// Accessing ==============================================================

	public boolean isUserReadable() {
		return userReadable;
	}

	public boolean isUserWritable() {
		return userWritable;
	}

	public boolean isUserExecutable() {
		return userExecutable;
	}

	public boolean isGroupReadable() {
		return groupReadable;
	}

	public boolean isGroupWritable() {
		return groupWritable;
	}

	public boolean isGroupExecutable() {
		return groupExecutable;
	}

	public boolean isWorldReadable() {
		return worldReadable;
	}

	public boolean isWorldWritable() {
		return worldWritable;
	}

	public boolean isWorldExecutable() {
		return worldExecutable;
	}

	public String toUnixString() {
		final StringBuilder buffer = new StringBuilder();
		buffer.append("u=");
		chmod(true, userWritable, userExecutable, buffer);
		buffer.append(",g=");
		chmod(groupReadable, groupWritable, groupExecutable, buffer);
		buffer.append(",o=");
		chmod(worldReadable, worldWritable, worldExecutable, buffer);
		return buffer.toString();
	}

	// Utils ==================================================================

	private void chmod(boolean readable, boolean writable, boolean executable, StringBuilder buffer) {
		if (readable) {
			buffer.append("r");
		}
		if (writable) {
			buffer.append("w");
		}
		if (executable) {
			buffer.append("x");
		}
	}
}
