package org.netbeans.lib.cvsclient.progress;

import org.netbeans.lib.cvsclient.util.*;

/**
 * @author Thomas Singer
 */
public final class RangeProgressViewer
		implements IProgressViewer {

	// Static =================================================================

	public static IProgressViewer createInstance(IProgressViewer parentProgressViewer, long currentIndex, long indexCount) {
		BugLog.assertNotNull(parentProgressViewer);
		BugLog.assertTrue(currentIndex >= 0, "");
		BugLog.assertTrue(currentIndex < indexCount, "");

		if (currentIndex == 0 && indexCount == 1) {
			return parentProgressViewer;
		}

		final double lowerBound = 1.0 * currentIndex / indexCount;
		final double upperBound = 1.0 * (currentIndex + 1) / indexCount;
		return new RangeProgressViewer(parentProgressViewer, lowerBound, upperBound);
	}

	// Fields =================================================================

	private final IProgressViewer parentProgressViewer;
	private final double lowerBound;
	private final double upperBound;

	// Setup ==================================================================

	public RangeProgressViewer(IProgressViewer parentProgressViewer, double lowerBound, double upperBound) {
		BugLog.assertNotNull(parentProgressViewer);

		this.parentProgressViewer = parentProgressViewer;
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}

	// Implemented ============================================================

	public void setProgress(double value) {
		final double boundedValue = (1.0 - value) * lowerBound + value * upperBound;
		parentProgressViewer.setProgress(boundedValue);
	}
}
