package org.netbeans.lib.cvsclient.progress.receiving;

import java.util.*;
import java.io.*;

import org.netbeans.lib.cvsclient.admin.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.progress.*;
import org.netbeans.lib.cvsclient.util.*;

/**
 * @author Thomas Singer
 */
public final class ResponseProgressHandler {

	// Static =================================================================

	public static Collection<FsFilePath> getFsFiles(ICvsFiles cvsFiles) {
		final Collection<FsFilePath> fsFiles = new ArrayList<FsFilePath>(2000);
		try {
			cvsFiles.visitFiles(new ICvsFilesVisitor() {
				public void handleFile(FsFilePath fsFile, Entry entry, boolean exists) {
					fsFiles.add(fsFile);
				}
			});
		}
		catch (CommandAbortedException e) {
			e.printStackTrace();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		return fsFiles;
	}

	// Fields =================================================================

	private final Set<FsFilePath> knownFsFiles = new HashSet<FsFilePath>(2000);
	private final Map<FsDirectoryPath, FsFilesCount> directoryPaths = new HashMap<FsDirectoryPath, FsFilesCount>(200);
	private final IProgressViewer progressViewer;
	private final int maxCount;
	private int count;
	private FsDirectoryPath previousDirectoryPath;

	// Setup ==================================================================

	public ResponseProgressHandler(IProgressViewer progressViewer, Collection<FsFilePath> fsFiles) {
		BugLog.assertNotNull(progressViewer);
		BugLog.assertNotNull(fsFiles);

		this.progressViewer = progressViewer;

		for (final FsFilePath fsFile : fsFiles) {
			knownFsFiles.add(fsFile);
			addDirectory(fsFile.getParent());
		}

		maxCount = this.knownFsFiles.size();
	}

	// Actions ================================================================

	public void fileProcessed(FsFilePath fsFile) {
		if (!knownFsFiles.remove(fsFile)) {
			return;
		}

		final FsFilesCount count = directoryPaths.get(fsFile.getParent());
		if (count == null) {
			return;
		}

		count.dec();

		notifyProgressViewer(1);
	}

	public void directoryProcessed(FsDirectoryPath directoryPath) {
		if (previousDirectoryPath != null && !previousDirectoryPath.equals(directoryPath)) {
			final FsFilesCount previousDirectoryCount = directoryPaths.remove(previousDirectoryPath);
			if (previousDirectoryCount != null) {
				notifyProgressViewer(previousDirectoryCount.getUnprocessedFilesInDirectory());
			}
		}

		previousDirectoryPath = directoryPath;
	}

	// Utils ==================================================================

	private void addDirectory(FsDirectoryPath directoryPath) {
		FsFilesCount count = directoryPaths.get(directoryPath);
		if (count == null) {
			count = new FsFilesCount();
			directoryPaths.put(directoryPath, count);
		}

		count.inc();
	}

	private void notifyProgressViewer(int incrementProgress) {
		if (incrementProgress == 0) {
			return;
		}

		count += incrementProgress;
		progressViewer.setProgress(maxCount > 0 ? (double)count / maxCount : Double.NaN);
	}

	// Inner classes ==========================================================

	private static final class FsFilesCount {

		// Fields =================================================================

		private int fsFilesPerDirectory;

		// Setup ==================================================================

		public FsFilesCount() {
		}

		// Implemented ============================================================

		public String toString() {
			return String.valueOf(fsFilesPerDirectory);
		}

		// Accessing ==============================================================

		private void inc() {
			fsFilesPerDirectory++;
		}

		private void dec() {
			fsFilesPerDirectory--;
		}

		private int getUnprocessedFilesInDirectory() {
			return fsFilesPerDirectory;
		}
	}
}
